Package: Rsubread
Version: 2.26.0
Date: 2026-03-17
Title: Mapping, quantification and variant analysis of sequencing data
Author: Wei Shi, Yang Liao and Gordon K Smyth with contributions from Jenny Dai
Maintainer: Wei Shi <wei.shi2@monash.edu>, Yang Liao <yang.liao@monash.edu> and Gordon K Smyth <smyth@wehi.edu.au>
Description: Alignment, quantification and analysis of
        RNA sequencing data (including both bulk RNA-seq and scRNA-seq) and
        DNA sequenicng data (including ATAC-seq, ChIP-seq, WGS, WES etc). 
        Includes functionality for read mapping, read counting, SNP calling, 
        structural variant detection and gene fusion discovery. Can be applied
        to all major sequencing techologies and to both short and long
        sequence reads.
URL: http://bioconductor.org/packages/Rsubread
License: GPL (>=3)
Imports: grDevices, stats, utils, Matrix
biocViews: Sequencing, Alignment, SequenceMatching, RNASeq, ChIPSeq,
        SingleCell, GeneExpression, GeneRegulation, Genetics,
        ImmunoOncology, SNP, GeneticVariability, Preprocessing,
        QualityControl, GenomeAnnotation, GeneFusionDetection,
        IndelDetection, VariantAnnotation, VariantDetection,
        MultipleSequenceAlignment
git_url: https://git.bioconductor.org/packages/Rsubread
git_branch: RELEASE_3_23
git_last_commit: 2c0bab4
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-04-28
NeedsCompilation: yes
Packaged: 2026-04-29 00:27:29 UTC; biocbuild
